/*	--*- c -*--
 * Copyright (C) 2014 Enrico Scholz <enrico.scholz@sigma-chemnitz.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "vita1300-remapper.h"

#include <time.h>
#include <stdlib.h>
#include <assert.h>
#include <stdint.h>
#include <stdio.h>

static uint64_t tm_uint64(struct timespec const *t)
{
	uint64_t	res = t->tv_sec;

	res *= 1000000000ul;
	res += t->tv_nsec;

	return res;
}

static char const *tm_delta(struct timespec const *a,
			    struct timespec const *b)
{
	static char		res[128];

	uint64_t		t_a = tm_uint64(a);
	uint64_t		t_b = tm_uint64(b);
	uint64_t		delta;

	if (t_a > t_b)
		delta = t_a - t_b;
	else
		delta = t_b - t_a;

	sprintf(res, "%lu.%09lu",
		(unsigned long)(delta / 1000000000ul),
		(unsigned long)(delta % 1000000000ul));

	return res;
}

static void test(size_t w, size_t h, enum vita1300_subsampling_mode mode, int bpp,
		 size_t cnt, int offs)
{
	size_t		len = (w+offs) * h * ((bpp + 7) / 8);
	void		*src = malloc(len);
	void		*dst = malloc(len);
	size_t		i;
	struct timespec	tm_a, tm_b, tm_c;


	assert(src != NULL);
	assert(dst != NULL);
	
	clock_gettime(CLOCK_MONOTONIC, &tm_a);
	for (i = 0; i < cnt; ++i) {
		int		rc;

		rc = vita1300_convert(src+offs, src+offs, mode, w, h, bpp);
		assert(rc == VITA1300_RES_OK);
	}

	clock_gettime(CLOCK_MONOTONIC, &tm_b);

	for (i = 0; i < cnt; ++i) {
		int		rc;

		rc = vita1300_convert(dst+offs, src+offs, mode, w, h, bpp);
		assert(rc == VITA1300_RES_OK);
	}

	clock_gettime(CLOCK_MONOTONIC, &tm_c);

	printf("%-5s/%2dbpp (%zu iterations, (%zu+%d)x%zu):",
	       mode == VITA1300_SUBSAMPLING_NONE ? "NONE" :
	       mode == VITA1300_SUBSAMPLING_MONO ? "MONO" :
	       mode == VITA1300_SUBSAMPLING_COLOR ? "COLOR" : "???",
	       bpp, cnt, w, offs, h);
	printf("\t%s",   tm_delta(&tm_b, &tm_a));
	printf("\t%s\n", tm_delta(&tm_c, &tm_b));

	free(dst);
	free(src);
}

#define T(_w, _h, _mode, _it)				\
	do {						\
		test(_w, _h, _mode,  8, _it, 0);		\
		test(_w, _h, _mode,  8, _it, 1);		\
		test(_w, _h, _mode, 10, _it, 0);	\
		test(_w, _h, _mode, 10, _it, 1);	\
	} while (0)

int main(void)
{
	size_t const	it = 10;

 	T(8192, 8192, VITA1300_SUBSAMPLING_NONE, it);
 	T(8192, 8192, VITA1300_SUBSAMPLING_MONO, it);
 	T(8192, 8192, VITA1300_SUBSAMPLING_COLOR, it);
}
