/*	--*- c -*--
 * Copyright (C) 2014 Enrico Scholz <enrico.scholz@sigma-chemnitz.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "vita1300-remapper.h"

#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <assert.h>

static unsigned int const	PATTERN_0_SRC[] = {
	0,   1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
	20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35
};

static unsigned int const	PATTERN_0_REF_NONE[] = {
	0,   4,  1,  5,  2,  6,  3,  7, 15, 11, 14, 10, 13,  9, 12,  8,
	20, 24, 21, 25, 22, 26, 23, 27, 35, 31, 34, 30, 33, 29, 32, 28,
};

static unsigned int const	PATTERN_0_REF_MONO[] = {
	0,   1,  2,  3,  7,  6,  5,  4,  8,  9, 10, 11, 15, 14, 13, 12,
	20, 21, 22, 23, 27, 26, 25, 24, 28, 29, 30, 31, 35, 34, 33, 32,
};


static unsigned int const	PATTERN_0_REF_COLOR[] = {
	0,   4,  2,  6,  7,  3,  5,  1,  8, 12, 10, 14, 15, 11, 13,  9,
	20, 24, 22, 26, 27, 23, 25, 21, 28, 32, 30, 34, 35, 31, 33, 29,
};

static void run_test_8(unsigned int const src[],
		       unsigned int const ref[],
		       enum vita1300_subsampling_mode mode,
		       unsigned int w, unsigned int h,
			int s_offs, int d_offs)
{
	size_t const		num_bits = 8;
	typedef uint8_t		pix_t;

#include "test-00.inc.h"
}

static void run_test_10(unsigned int const src[],
			unsigned int const ref[],
			enum vita1300_subsampling_mode mode,
			unsigned int w, unsigned int h,
			int s_offs, int d_offs)
{
	size_t const		num_bits = 10;
	typedef uint16_t	pix_t;

#include "test-00.inc.h"

}

#define ARRAY_SIZE(_a) ((sizeof(_a)) / (sizeof(_a)[0]))

#define T0(_idx, _mode, _rows, _bpp, _s_offs, _d_offs)			\
	do {								\
		_Static_assert(ARRAY_SIZE(PATTERN_ ## _idx ## _SRC) ==	\
			       ARRAY_SIZE(PATTERN_ ## _idx ## _REF_ ## _mode), \
			       "pattern array mismatch");		\
									\
		_Static_assert(ARRAY_SIZE(PATTERN_ ## _idx ## _SRC) %	\
			       (_rows) == 0, "odd row number");		\
									\
		run_test_ ## _bpp(PATTERN_ ## _idx ## _SRC,		\
				  PATTERN_ ## _idx ## _REF_ ## _mode,	\
				  VITA1300_SUBSAMPLING_ ## _mode,	\
				  ARRAY_SIZE(PATTERN_ ## _idx ## _SRC) / (_rows), \
				  (_rows), (_s_offs), (_d_offs));	\
	} while (0)

#define T1(_idx, _mode, _rows, _bpp)			\
	do {						\
		T0(_idx, _mode, _rows, _bpp, 0, 0);	\
		T0(_idx, _mode, _rows, _bpp, 0, 1);	\
		T0(_idx, _mode, _rows, _bpp, 1, 0);	\
		T0(_idx, _mode, _rows, _bpp, 1, 1);	\
		T0(_idx, _mode, _rows, _bpp, 8, 7);	\
		T0(_idx, _mode, _rows, _bpp, 8, 8);	\
	} while (0)

#define T2(_idx, _mode, _rows)			\
	do {					\
		T1(_idx, _mode, _rows, 8);	\
		T1(_idx, _mode, _rows, 10);	\
	} while (0)

#define T3(_idx, _rows)				\
	do {					\
		T2(_idx, NONE, _rows);		\
		T2(_idx, MONO, _rows);		\
		T2(_idx, COLOR, _rows);		\
	} while (0)

#define T(_idx)					\
	T3(_idx, 2)

int main(void)
{
	T(0);
}
