/*	--*- c -*--
 * Copyright (C) 2014 Enrico Scholz <enrico.scholz@sigma-chemnitz.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.
 */

#include "vita1300-remapper.h"

#include <stdint.h>
#include <string.h>

#define likely(_cond)		(__builtin_expect(!!(_cond),1))

static void vita1300_convert_none_8(void *dst, void const *src,
				    unsigned int w, unsigned int h)
{
	typedef uint8_t		pix_t;

#include <convert-none.h>
}

static void vita1300_convert_none_16(void *dst, void const *src,
				     unsigned int w, unsigned int h)
{
	typedef uint16_t	pix_t;

#include <convert-none.h>
}


static void vita1300_convert_mono_8(void *dst, void const *src,
				    unsigned int w, unsigned int h)
{
	typedef uint8_t		pix_t;

#include <convert-mono.h>
}

static void vita1300_convert_mono_16(void *dst, void const *src,
				     unsigned int w, unsigned int h)
{
	typedef uint16_t	pix_t;

#include <convert-mono.h>
}



static void vita1300_convert_color_8(void *dst, void const *src,
				     unsigned int w, unsigned int h)
{
	typedef uint8_t		pix_t;

#include <convert-color.h>
}

static void vita1300_convert_color_16(void *dst, void const *src,
				      unsigned int w, unsigned int h)
{
	typedef uint16_t	pix_t;

#include <convert-color.h>
}

enum vita1300_result __attribute__((__visibility__("default")))
vita1300_convert(void *dst, void const *src,
		 enum vita1300_subsampling_mode mode,
		 unsigned int w, unsigned int h,
		 unsigned int bpp)
{
	size_t		l = w * h * ((bpp + 7) / 8);

	if (bpp < 8 || bpp > 16)
		return VITA1300_BAD_BPP;

	if (((src > dst) && (src < dst + l)) ||
	    ((dst > src) && (dst < src + l)))
		return VITA1300_BAD_OVERLAP;

	switch (mode) {
	case VITA1300_SUBSAMPLING_NONE:
		if (w % 16 != 0)
			return VITA1300_BAD_X;

		if (bpp == 8)
			vita1300_convert_none_8(dst, src, w, h);
		else
			vita1300_convert_none_16(dst, src, w, h);
		break;

	case VITA1300_SUBSAMPLING_MONO:
		if (w % 8 != 0)
			return VITA1300_BAD_X;

		if (bpp == 8)
			vita1300_convert_mono_8(dst, src, w, h);
		else
			vita1300_convert_mono_16(dst, src, w, h);
		break;

	case VITA1300_SUBSAMPLING_COLOR:
		if (w % 8 != 0)
			return VITA1300_BAD_X;

		if (bpp == 8)
			vita1300_convert_color_8(dst, src, w, h);
		else
			vita1300_convert_color_16(dst, src, w, h);
		break;

	default:
		return VITA1300_BAD_MODE;
	}

	return VITA1300_RES_OK;
}
